use nostr::{Event, Filter, Kind, Tag};

fn ids_match(filter: &Filter, event: &Event) -> bool {
    if filter.ids.is_empty() {
        println!("[FILTER][IDS] skipped");
        return true;
    }

    println!(
        "[FILTER][IDS] matched: {:?}",
        filter.ids.iter().any(|id| id == &event.id.to_string())
    );

    filter.ids.iter().any(|id| id == &event.id.to_string())
}

fn kind_match(filter: &Filter, kind: Kind) -> bool {
    if filter.kinds.is_empty() {
        println!("[FILTER][KINDS] skipped");
        return true;
    }

    println!(
        "[FILTER][KIND] matched: {:?}",
        filter.kinds.iter().any(|k| k == &kind)
    );

    filter.kinds.iter().any(|k| k == &kind)
}

fn pubkeys_match(filter: &Filter, event: &Event) -> bool {
    if filter.pubkeys.is_empty() {
        println!("[FILTER][PUBKEYS] skipped");
        return true;
    }

    println!(
        "[FILTER][PUBKEYS] matched: {:?}",
        filter.pubkeys.iter().any(|pk| pk == &event.pubkey)
    );
    filter.pubkeys.iter().any(|pk| pk == &event.pubkey)
}

fn authors_match(filter: &Filter, event: &Event) -> bool {
    dbg!(filter);
    if filter.authors.is_empty() {
        println!("[FILTER][AUTHORS] skipped");
        return true;
    }

    println!(
        "[FILTER][AUTHORS] matched: {:?}",
        filter
            .authors
            .iter()
            .any(|author| author == &event.pubkey.to_string())
    );
    filter
        .authors
        .iter()
        .any(|author| author == &event.pubkey.to_string())
}

fn delegated_authors_match(filter: &Filter, event: &Event) -> bool {
    // Optional implementation

    // let delegated_authors_match = filter.authors.iter().any(|author| {
    //     event.tags.iter().any(|tag| match tag {
    //         Tag::Delegation {
    //             delegator_pk,
    //             conditions,
    //             sig,
    //         } => filter
    //             .authors
    //             .iter()
    //             .any(|author| author == &delegator_pk.to_string()),
    //         _ => false,
    //     })
    // });
    println!(
        "[FILTER][DELEGATED_AUTHORS] matched: {:?}",
        event.tags.iter().any(|tag| match tag {
            Tag::Delegation {
                delegator_pk,
                conditions,
                sig,
            } => filter
                .authors
                .iter()
                .any(|author| author == &delegator_pk.to_string()),
            _ => false,
        })
    );

    event.tags.iter().any(|tag| match tag {
        Tag::Delegation {
            delegator_pk,
            conditions,
            sig,
        } => filter
            .authors
            .iter()
            .any(|author| author == &delegator_pk.to_string()),
        _ => true,
    })
}

fn tag_match(filter: &Filter, event: &Event) -> bool {
    println!(
        "[FILTER][TAG] matched: {:?}",
        filter.generic_tags.iter().any(|(key, value)| {
            event.tags.iter().any(|tag| {
                let kv = tag.as_vec();
                key.to_string() == kv[0] && value.iter().any(|vv| vv == &kv[1])
            })
        })
    );

    filter.generic_tags.iter().any(|(key, value)| {
        event.tags.iter().any(|tag| {
            let kv = tag.as_vec();
            key.to_string() == kv[0] && value.iter().any(|vv| vv == &kv[1])
        })
    });

    true // TODO: Fix delegated authors check
}

pub fn interested_in_event(filter: &Filter, event: &Event) -> bool {
    ids_match(filter, event)
        && filter.since.map_or(
            {
                println!("[FILTER][SINCE][default] matched: {:?}", true);
                true
            },
            |t| {
                println!("[FILTER][SINCE] matched: {:?}", event.created_at >= t);
                event.created_at >= t
            },
        )
        && filter.until.map_or(
            {
                println!("[FILTER][UNTIL][default] matched: {:?}", true);
                true
            },
            |t| {
                println!("[FILTER][UNTIL] matched: {:?}", event.created_at <= t);
                event.created_at <= t
            },
        )
        && kind_match(filter, event.kind)
        && (pubkeys_match(filter, event)
            || authors_match(filter, event)
            || delegated_authors_match(filter, event))
        && tag_match(filter, event)
}
